"""
Hieronder de helper functies. Elke functie **moet** een docstring hebben waarin de parameters en return value worden
beschreven.
"""

import rasterio as rio
import shapely
import geopandas as gp
import affine
import numpy as np
import pandas as pd
import random
import numbers
import os
import argparse


def rand_web_color_hex():
    """ "
    Generate random color.
    https://blog.stigok.com/2019/06/22/generate-random-color-codes-python.html
    """
    rgb = ""
    for _ in "RGB":
        i = random.randrange(0, 2 ** 8)
        rgb += i.to_bytes(1, "big").hex()
    return "#{}".format(rgb)


def pnt2square_id(east_north, size, of="full"):
    """
    Used to slot a point into a geographical square based on its coordinates.

    :param east_north: tuple easting, northing coordinates in EPSG3035
    :param size: length in m of the squares (eg 10.000 or 25.000 m)
    :param of: brief or full output format, default full
    :return: square identification string as eeee_nnnn: E-N of square lower left coordinates if full, else eeennnn

        |
        |
      N |        X
        |
        @_____________
              E
        <------------>
             size

     Given plot at location X and square size x size, return Easting Northing of @ as "eeee_nnnn"
    """
    easting, northing = east_north

    ll_easting = np.multiply(
        np.floor_divide(easting, size), size
    )  # easting of square lower left
    ll_northing = np.multiply(
        np.floor_divide(northing, size), size
    )  # northing of square lower left
    out = {
        "full": "{:04d}_{:04d}".format(int(ll_easting), int(ll_northing)),
        "brief": np.add(ll_easting, np.divide(ll_northing, 1000)).astype(np.uint32),
    }

    try:
        return out[of]
    except KeyError:
        print("select either full or brief as output format")
        raise


def gen_squares(x_ul, y_ul, nrow, ncol, size):
    """
    Generate geodataframe with square polygons
    :param x_ul: easting top-left corner
    :param y_ul: northing top-left corner
    :param nrow: number of rows
    :param ncol: number of cols
    :param size: square size in m
    :return: gdf with ncol*nrow items
    """

    """
    Definition affine.Affine(a, b, c, d, e, f)
      a: width of a pixel
      b: row rotation (typically zero)
      c: x-coordinate of the upper-left of the upper-left pixel
      d: column rotation (typically zero)
      e: height of a pixel (negative)
      f: y-coordinate of the upper-left of the upper-left pixel
    https://www.perrygeo.com/python-affine-transforms.html
    """

    aff = affine.Affine(size, 0, x_ul, 0, -size, y_ul)
    ul_corners_indx = [indx for indx in np.ndindex(ncol, nrow)]
    ul_corners_coords = [
        colrow2xy(colrow=corner, affine_in=aff) for corner in ul_corners_indx
    ]
    # [(xmin, ymin), (xmax, ymin), (xmax, ymax), (xmin, ymax)] == [ll, lr, ur, ul]
    all_corner_coords = [
        [
            (xmin, ymax - size),
            (xmin + size, ymax - size),
            (xmin + size, ymax),
            (xmin, ymax),
        ]
        for (xmin, ymax) in ul_corners_coords
    ]
    shapes = [shapely.geometry.Polygon(corner) for corner in all_corner_coords]

    ids = ["{0}_{1}".format(int(x), int(y)) for x, y in ul_corners_coords]
    return gp.GeoDataFrame(
        geometry=shapes,
        data={"ID": ids, "size": str(size)},
        crs={
            "x_0": 155000,
            "proj": "sterea",
            "units": "m",
            "y_0": 463000,
            "k": 0.9999079,
            "lon_0": 5.38763888888889,
            "no_defs": True,
            "lat_0": 52.15616055555555,
            "ellps": "bessel",
        },
    )


def colrow2xy(colrow, rio_object=None, affine_in=None):
    """
    https://www.perrygeo.com/python-affine-transforms.html
    :param colrow: column-index_row-index tuple
    :param rio_object: rio object
    :param affine_in: affine transformation
    :return: easting norhting in CRS
    """
    try:
        a = rio_object.affine
    except AttributeError:
        if isinstance(affine_in, affine.Affine):
            a = affine_in
        else:
            raise Exception(
                "{0} is not a valid rasterio object and no Affine alternative supplied".format(
                    rio_object
                )
            )
    col, row = colrow
    x, y = a * (col, row)
    return x, y



def progress_bar(iterable, prefix='', suffix='', decimals=1, length=100, fill='#', printEnd="\r"):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
        printEnd    - Optional  : end character (e.g. "\r", "\r\n") (Str)
    credits: https://stackoverflow.com/questions/3173320/text-progress-bar-in-the-console
    """
    total = len(iterable)

    def printProgressBar(iteration):
        percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
        filledLength = int(length * iteration // total)
        bar = fill * filledLength + '-' * (length - filledLength)
        print('\r{0} |{1}| {2}% {3}'.format(prefix, bar, percent, suffix), end=printEnd)

    # Initial Call
    printProgressBar(0)

    # Update Progress Bar
    for i, item in enumerate(iterable):
        yield item
        printProgressBar(i + 1)
    # Print New Line on Complete
    print()


def geospatial_raster(x):
    """
    Is x a geospatial raster?
    :param x: file path
    :return: x if True else argparse.ArgumentTypeError
    """

    if os.path.isfile(x):
        try:
            rast = rio.open(x)
            return x
        except(rio.errors.RasterioIOError, rio.errors.FileError, rio._err.CPLE_OpenFailed) as e:
            raise argparse.ArgumentTypeError(e)
    else:
        raise argparse.ArgumentTypeError('Not a file: {0}'.format(x))


def readable_dir(prospective_dir):
    """
    Check is a prospective dir is valid and readable.
    
    :param prospective_dir: directory path
    :return: prospective dir or argparse.ArgumentTypeError
    """
    if not os.path.isdir(prospective_dir):
        raise argparse.ArgumentTypeError("readable_dir:{0} is not a valid path".format(prospective_dir))
    if os.access(prospective_dir, os.R_OK):
        return prospective_dir
    else:
        raise Exception("readable_dir:{0} is not a readable dir".format(prospective_dir))
        
        
def file_of_type(x, ext):
    """
    Is x a file of type ext?
    :param x: file path
    :param ext: extension (without dot)
    :return: x if x exists and has required extension, else argparse TypeError
    """
    
    exists = os.path.isfile(x)
    ext_match = os.path.splitext(os.path.basename(x))[1] == '.{}'.format(ext)
    if exists and ext_match:
        return x
    if not exists:
        raise argparse.ArgumentTypeError('{} is not a file'.format(x))
    if not ext_match:
        raise argparse.ArgumentTypeError('{} is not a correct file'.format(x))

